@@ -1,4 +1,4 @@
-import os
+from os.path import join
 from pythonforandroid.recipe import PythonRecipe
 
 
@@ -15,7 +15,7 @@ class ZBarRecipe(PythonRecipe):
 
     call_hostpython_via_targetpython = False
 
-    depends = ['hostpython2', 'python2', 'setuptools', 'libzbar']
+    depends = ['setuptools', 'libzbar']
 
     patches = ["zbar-0.10-python-crash.patch"]
 
@@ -24,13 +24,9 @@ class ZBarRecipe(PythonRecipe):
         libzbar = self.get_recipe('libzbar', self.ctx)
         libzbar_dir = libzbar.get_build_dir(arch.arch)
         env['PYTHON_ROOT'] = self.ctx.get_python_install_dir()
-        env['CFLAGS'] += ' -I' + os.path.join(libzbar_dir, 'include')
-        env['CFLAGS'] += ' -I' + env['PYTHON_ROOT'] + '/include/python2.7'
-        # TODO
-        env['LDSHARED'] = env['CC'] + \
-            ' -pthread -shared -Wl,-O1 -Wl,-Bsymbolic-functions'
-        # TODO: hardcoded Python version
-        env['LDFLAGS'] += " -landroid -lpython2.7 -lzbar"
+        env['CFLAGS'] += ' -I' + join(libzbar_dir, 'include')
+        env['LDFLAGS'] += ' -L' + join(libzbar_dir, 'zbar', '.libs')
+        env['LIBS'] = env.get('LIBS', '') + ' -landroid -lzbar'
         return env
 
 
